
//-------------------------------------------------------------------
//
// TITLE:   Tanks battle simulation / Tutorial
// VERSION: 1.0 - Chapter 3
// AUTHOR:  Avi Chami
// DATE:    11/05/2002
//
//-------------------------------------------------------------------


PROGRAM tankbat3;


// ******************************************************************
//
//  The chapters of the tutorial will show you how to make a simple
//  tank combat simulation, on 2D (top view).
//
//  This tutorial will show you some techniques that can
//  be used later to make an RTS or TBS program
//  (Real Time or Turn Based Strategy)
//  On each chapter of the Tutorial, I will include detailed comments
//  for the added portions of the code
//
//
//  Hope you will find them helpful
//
//  You can contact me at:
//        chclau@yahoo.com
//
// ******************************************************************


// ******************************************************************
//  In this third chapter, we will add the tanks from the enemy group,
//  add the 'shields' (more on this later), and the firing of 'group 2'
//  tanks, until all 'group 1' tanks are destroyed
// ******************************************************************


//-------------------------------------------------------------------
CONST

  MAX_TANKS   = 90;             // Maximum number of tanks on each side
                                // Defines the size of the array that
                                // contains the IDs of the tank processes


  // Quantity of tanks in each group - Must be <= 90
  TANK1_NO    = 40;
  TANK2_NO    = 10;

  // This two parameters represent the 'health' of each type of tanks
  // (that is, how many hits has to take a tank to be knocked down)
  // Change here to see their influence in the result of the battle
  SHIELD1     =  1;
  SHIELD2     =  10;


//-------------------------------------------------------------------
GLOBAL

// Graphics and sound

  fpg_id;                       // Id of the fpg file



// Tanks group info

  tanks1_id[MAX_TANKS];         // Array with the IDs of all the
  tanks2_id[MAX_TANKS];         // tanks processes

  tanks1_shield[MAX_TANKS];     // Remaining shield for each tank in the
  tanks2_shield[MAX_TANKS];     // group - When shield is 0 -> destroy tank

  tanks1_qtty;                  // Qtty of tanks in group one
  tanks2_qtty;                  // Qtty of tanks in group two



// General purpose

  g_ind;                        // General purpose indexes
  g_ind2;
  g_ind3;


BEGIN

//-------------------------------------------------------------------
// Initialize sound, graphics, board
//-------------------------------------------------------------------

    max_process_time = 2000;

    set_mode(m1024x768);        // Sets video mode to 1024x768
                                // Since we are going to draw a lot of
                                // tamks - I chose the biggest resolution

    set_fps(24,0);              // Number of frames per second


// Load graphics file
    fpg_id = load_fpg("mytut\tankbat.fpg");



// Initialize game variables

    tanks1_qtty = TANK1_NO;
    tanks2_qtty = TANK2_NO;




//-------------------------------------------------------------------
// Draw all the tanks of the first group, in columns of 15 tanks
// g_ind1 - Counts vertically from 0 to 14 on each column
// g_ind2 - Remembers the qtty of drawn tanks so far
// g_ind3 - Used to draw the tanks in columns of 15


// The first tank on group 1 will be drawn in the upper left corner
// Then, 15 tanks will be drawn in descending order in the first column
// Tank #16 will be drawn on top of the 2nd column, and so on.
// Tank #90 will be drawn at the bottom of the last column


// Note that the shields are initialized for each tank inside the loops

    g_ind2 = 1;

    FROM g_ind3 = 0 TO 2;

        FROM g_ind = 0 TO 14;

            IF (g_ind2 > tanks1_qtty)
                break;
            END

            tanks1_id[g_ind2]     = tank1 (40 + g_ind3 * 120, 45 + g_ind * 46);
            tanks1_shield[g_ind2] = SHIELD1;
            g_ind2++;

        END


        FROM g_ind = 0 TO 14;

            IF (g_ind2 > tanks1_qtty)
                break;
            END

            tanks1_id[g_ind2]     = tank1 (100 + g_ind3 * 120, 65 + g_ind * 46);
            tanks1_shield[g_ind2] = SHIELD1;
            g_ind2++;

        END
    END


//-------------------------------------------------------------------
// Draw all the tanks on the second group, in columns of 15 tanks

// The first tank on group 2 will be drawn in the upper right corner
// Then, 15 tanks will be drawn in descending order in the last column
// Tank #16 will be drawn on top of the column to the left or the last.
// Tank #90 will be drawn at the bottom of the first column


    g_ind2      = 1;

    FROM g_ind3 = 0 TO -2 STEP -1;

        FROM g_ind = 0 TO 14;

            IF (g_ind2 > tanks2_qtty)
                break;
            END

            tanks2_id[g_ind2]     = tank2 (880 + g_ind3 * 140, 45 + g_ind * 46);
            tanks2_shield[g_ind2] = SHIELD2;
            g_ind2++;

        END


        FROM g_ind = 0 TO 14;

            IF (g_ind2 > tanks2_qtty)
                break;
            END

            tanks2_id[g_ind2]     = tank2 (810 + g_ind3 * 140, 65 + g_ind * 46);
            tanks2_shield[g_ind2] = SHIELD2;
            g_ind2++;

        END
    END




//-------------------------------------------------------------------
// Main loop
//
//   In the main loop we will fire the tanks from each group -
//   In this lesson we will only fire 'group 2' tanks
//-------------------------------------------------------------------

    LOOP

//  Fire tanks from group two

        FOR (g_ind = 1; g_ind <= tanks2_qtty; g_ind++)

            // Show the turret firing !
            // The 'recoil' effect is done by the special definition
            // of controil point 'zero' on each one of the graphs (24 to 26)

            FROM g_ind2 = 24 TO 26;

            // Each tank process son is the turret
            // So we are animating the turret of each tank

                tanks2_id[g_ind].son.graph = g_ind2;
                FRAME;
                FRAME;
            END

            tanks2_id[g_ind].son.graph = 22;


            // Enemy tank hit - decrement its shields
            tanks1_shield[tanks1_qtty]--;


            IF (tanks1_shield[tanks1_qtty] <= 0)

                // Tank destroyed - Kill the tank AND the turret

                signal (tanks1_id[tanks1_qtty], s_kill_tree);

                // Decrement tanks1 quantity
                tanks1_qtty--;

                // Small pause
                FRAME (1000);


            END

            // If all enemy tanks are destroyed - enter an endless loop
            WHILE (tanks1_qtty <= 0 )
                FRAME;
            END

            // Small pause
            FRAME(fps*30);

        END

        FRAME;

    END

END



//-------------------------------------------------------------------
// Process tank1
// Draws the basis of a green tank
//-------------------------------------------------------------------


PROCESS tank1 (x, y)

PRIVATE

BEGIN

    z     = 0;
    graph = 10;

// The turret is called from the tank1 process. So 'turret1' is
// the son, and tank1 is the father process.

    turret1 (x+1, y);

    LOOP
        FRAME;
    END

END


//-------------------------------------------------------------------
// Process turret1
// Draws the turret of a green tank
//-------------------------------------------------------------------


PROCESS turret1 (x, y)

PRIVATE

BEGIN

// The turret has lower 'z' than the tank, so it will be always drawn
// OVER the tank

    z     = -100;
    graph = 12;

    LOOP
        FRAME;
    END

END


//-------------------------------------------------------------------
// Process tank2
// Draws the basis of a green tank
// Similar to tank1 - Only difference is orientation (angle = 180)
//-------------------------------------------------------------------


PROCESS tank2 (x, y)

PRIVATE

BEGIN

    z = 0;
    graph = 20;
    angle = 180000;

    turret2 (x-1, y+1);

    LOOP
        FRAME;
    END

END




//-------------------------------------------------------------------
// Process turret2
// Draws the turret of a gray tank
//-------------------------------------------------------------------


PROCESS turret2 (x, y)

PRIVATE

BEGIN

    z = -100;
    graph = 22;
    angle = 180000;

    LOOP
        FRAME;
    END

END

