
//-------------------------------------------------------------------
//
// TITLE:   Tanks battle simulation / Tutorial
// VERSION: 1.0 - Chapter 6
// AUTHOR:  Avi Chami
// DATE:    11/05/2002
//
//-------------------------------------------------------------------


PROGRAM tankbat6;


// ******************************************************************
//
//  The chapters of the tutorial will show you how to make a simple
//  tank combat simulation, on 2D (top view).
//
//  This tutorial will show you some techniques that can
//  be used later to make an RTS or TBS program
//  (Real Time or Turn Based Strategy)
//  On each chapter of the Tutorial, I will include detailed comments
//  for the added portions of the code
//
//
//  Hope you will find them helpful
//
//  You can contact me at:
//        chclau@yahoo.com
//
// ******************************************************************


// ******************************************************************
//  In this sixth and last chapter, we will add some on screen
//  information to the ongoing simulation.
// ******************************************************************

// ******************************************************************
//  Summary of all the chapters:
//
//    We drew two groups of tanks in battle.
//    The quantity on each group can be of 1 to 90 tanks.
//    Each tank consists of four processes - the base and its shadow,
//    and the turret and its shadow.
//    Each tank has a 'shield' value: It means many hits it can take.
//    Each group fires on all the tanks of the other group, by turns,
//    until there is a winneril they killed them all.
//    We added sounds and animations for firing, explosions, etc.
//    We showed information on screen regarding the on going simulation.
//
//  The future:
//
//     When and if I continue this tutorial, I plan to go to a totally
//     different direction:
//     This was a very simple simulation in terms of the behaviour of
//     the tanks. In a future version I will add:
//
//     Tanks range, and movement, for a tank with no enemies in range
//     will advance till it finds one.
//     Crew experience.
//     Crew moral - It is unlikely that a crew seeing that tens of tanks
//     from his side are being destroyed, will continue to fight
//     as if nothing happened.
//     Etc., etc.
//
//     Of course you are invited to contribute with remarks, ideas, & code
// ******************************************************************

//-------------------------------------------------------------------
CONST

  MAX_TANKS   = 90;             // Maximum number of tanks on each side
                                // Defines the size of the array that
                                // contains the IDs of the tank processes
  MAX_ROUNDS  = 30;             // Maximum number of rounds on a match


  // Quantity of tanks in each group - Must be <= 90
  TANK1_NO    = 70;
  TANK2_NO    = 28;


  // This two parameters represent the 'health' of each type of tanks
  // (that is, how many hits has to take a tank to be knocked down)
  // Change here to see their influence in the result of the battle
  SHIELD1     =  2;
  SHIELD2     =  10;

  // Increment on angle for turret rotation while aiming to enemy tank
  // If this number is bigger - rotation will be faster
  ROTATE_ANGLE = 1200;


//-------------------------------------------------------------------
GLOBAL

// Graphics and sound

  fpg_id;                       // Id of the fpg file

  turret_snd;                   // Turret turning
  turret_channel;

  cannon_snd;                   // Cannon firing

  tank_expl_snd;                // Tank explodes


// Tanks group info

  tanks1_id[MAX_TANKS];         // Array with the IDs of all the
  tanks2_id[MAX_TANKS];         // tanks processes

  tanks1_shield[MAX_TANKS];     // Remaining shield for each tank in the
  tanks2_shield[MAX_TANKS];     // group - When shield is 0 -> destroy tank

  tanks1_qtty;                  // Qtty of tanks in group one
  tanks2_qtty;                  // Qtty of tanks in group two



// Display variables

  disp_shield1;                 // Used to show tank group 1 shield value
  disp_shield2;                 // Used to show tank group 2 shield value

  round;                        // round number

  turn[MAX_ROUNDS];
  tank1_remain[MAX_ROUNDS];     // remaining tank group1 tanks on each round
  tank2_remain[MAX_ROUNDS];     // remaining tank group2 tanks on each round



// General purpose

  g_ind;                        // General purpose indexes
  g_ind2;
  g_ind3;

  angle2;                       // Used for aiming the tanks' turrets


BEGIN

//-------------------------------------------------------------------
// Initialize sound, graphics, board
//-------------------------------------------------------------------

    max_process_time = 2000;

    set_mode(m1024x768);        // Sets video mode to 1024x768
                                // Since we are going to draw a lot of
                                // tamks - I chose the biggest resolution

    set_fps(24,0);              // Number of frames per second


// Load graphics file
    fpg_id = load_fpg("mytut\tankbat.fpg");


// Load sounds
    turret_snd = load_pcm("library\machines\engines\motor2.pcm", 1);
    cannon_snd = load_pcm("library\war\shoots\disparo7.pcm", 0);
    tank_expl_snd = load_pcm("library\war\explosio\explos00.pcm", 0);


// Put background
    put_screen (fpg_id, 100);   // On the right of the screen, we left
                                // space for displaying info
                                // The tanks gfx were addapted from Adherbal
                                // game WWII
                                // The background gfx are from 'Alien' game
                                // (from DIV sample progs)
                                // The wood texture is from POVRAY


// Initialize game variables

    tanks1_qtty = TANK1_NO;
    tanks2_qtty = TANK2_NO;


//
// Show game information on screen
//

   write (0, 972, 700, 4, "Tank Battle 1.0");
   write (0, 972, 730, 4, "Graphics   by");
   write (0, 972, 745, 4, "Lukas Nijsten /");
   write (0, 972, 757, 4, "Avi Chami");




   write (0, 970, 30, 4, "Shields");
   write (0, 970, 50, 4, "T1        T2");

   disp_shield1 = SHIELD1;       // Used to show tank group 1 shield value
   disp_shield2 = SHIELD2;       // Used to show tank group 2 shield value

   write_int (0, 945, 65, 5, OFFSET disp_shield1);
   write_int (0, 1005, 65, 5, OFFSET disp_shield2);


   write (0, 970, 110, 4, "Remaining");
   write (0, 970, 130, 4, "T1        T2");

   write_int (0, 945, 145, 5, OFFSET tanks1_qtty);
   write_int (0, 1005, 145, 5, OFFSET tanks2_qtty);


   round = 0;
   turn[0]= round;
   tank1_remain[0] = tanks1_qtty;
   tank2_remain[0] = tanks2_qtty;

   write (0, 930, 200, 3, "Round  T1   T2");
   write_int (0, 950, 215, 5, OFFSET turn[0]);
   write_int (0, 985, 215, 5, OFFSET tank1_remain[0]);
   write_int (0, 1015, 215, 5, OFFSET tank2_remain[0]);


//-------------------------------------------------------------------
// Draw all the tanks of the first group, in columns of 15 tanks
// g_ind1 - Counts vertically from 0 to 14 on each column
// g_ind2 - Remembers the qtty of drawn tanks so far
// g_ind3 - Used to draw the tanks in columns of 15


// The first tank on group 1 will be drawn in the upper left corner
// Then, 15 tanks will be drawn in descending order in the first column
// Tank #16 will be drawn on top of the 2nd column, and so on.
// Tank #90 will be drawn at the bottom of the last column


// Note that the shields are initialized for each tank inside the loops

    g_ind2 = 1;

    FROM g_ind3 = 0 TO 2;

        FROM g_ind = 0 TO 14;

            IF (g_ind2 > tanks1_qtty)
                break;
            END

            tanks1_id[g_ind2]     = tank1 (40 + g_ind3 * 120, 45 + g_ind * 46);
            tanks1_shield[g_ind2] = SHIELD1;
            g_ind2++;

        END


        FROM g_ind = 0 TO 14;

            IF (g_ind2 > tanks1_qtty)
                break;
            END

            tanks1_id[g_ind2]     = tank1 (100 + g_ind3 * 120, 65 + g_ind * 46);
            tanks1_shield[g_ind2] = SHIELD1;
            g_ind2++;

        END
    END


//-------------------------------------------------------------------
// Draw all the tanks on the second group, in columns of 15 tanks

// The first tank on group 2 will be drawn in the upper right corner
// Then, 15 tanks will be drawn in descending order in the last column
// Tank #16 will be drawn on top of the column to the left or the last.
// Tank #90 will be drawn at the bottom of the first column


    g_ind2      = 1;

    FROM g_ind3 = 0 TO -2 STEP -1;

        FROM g_ind = 0 TO 14;

            IF (g_ind2 > tanks2_qtty)
                break;
            END

            tanks2_id[g_ind2]     = tank2 (880 + g_ind3 * 140, 45 + g_ind * 46);
            tanks2_shield[g_ind2] = SHIELD2;
            g_ind2++;

        END


        FROM g_ind = 0 TO 14;

            IF (g_ind2 > tanks2_qtty)
                break;
            END

            tanks2_id[g_ind2]     = tank2 (810 + g_ind3 * 140, 65 + g_ind * 46);
            tanks2_shield[g_ind2] = SHIELD2;
            g_ind2++;

        END
    END


// Small pause - 4 sec

    FRAME (400 * fps);



//-------------------------------------------------------------------
// Main loop
//
//   In the main loop we will fire the tanks from each group -
//   In this lesson we will only fire 'group 2' tanks
//-------------------------------------------------------------------

    LOOP

//  Fire tanks from group two

        FOR (g_ind = 1; g_ind <= tanks2_qtty; g_ind++)


            // Rotate the turret to aim to the current enemy

            angle2 = fget_angle ( tanks2_id[g_ind].x, tanks2_id[g_ind].y,
                                      tanks1_id[tanks1_qtty].x, tanks1_id[tanks1_qtty].y);


            // Gradually change the turret angle to point to the enemy tank
            // Include a sound for the turret rotation

            turret_channel = sound (turret_snd, 512, 255);

            WHILE ( abs (tanks2_id[g_ind].son.angle - angle2) > ROTATE_ANGLE)

                tanks2_id[g_ind].son.angle = near_angle (tanks2_id[g_ind].son.angle, angle2, ROTATE_ANGLE);
                FRAME;
            END

            // Stop sound rotation sound
            stop_sound (turret_channel);
            FRAME;


            // Show the turret firing !
            // The 'recoil' effect is done by the special definition
            // of controil point 'zero' on each one of the graphs (24 to 26)

            // Play cannon fire sound

            sound(cannon_snd, 90, 192);

            FROM g_ind2 = 24 TO 26;

            // Each tank process son is the turret
            // So we are animating the turret of each tank

                tanks2_id[g_ind].son.graph = g_ind2;
                FRAME;
                FRAME;
            END

            tanks2_id[g_ind].son.graph = 22;


            // Enemy tank hit - decrement its shields
            tanks1_shield[tanks1_qtty]--;


            IF (tanks1_shield[tanks1_qtty] <= 0)


                // Tank destroyed - Draw big explosion

                sound (tank_expl_snd, 255, 255);

                big_explosion (tanks1_id[tanks1_qtty].x,
                                   tanks1_id[tanks1_qtty].y );



                // Kill the tank, the turret and the shadow

                signal (tanks1_id[tanks1_qtty], s_kill_tree);

                FRAME (1000);

                tanks1_qtty--;

            ELSE

                // Tanks not destroyed - draw 'hit'
                // The random elements for x and y show the explosions
                // hitting different points of the enemy tank, making
                // the hits more credible than if they were all on its
                // center.

                explosion (tanks1_id[tanks1_qtty].x + rand(-14,14),
                            tanks1_id[tanks1_qtty].y + rand(-7,7) );


            END

            // If all enemy tanks are destroyed - enter an endless loop
            WHILE (tanks1_qtty <= 0 )
                FRAME;
            END

            // Small pause
            FRAME(fps*30);

        END


// Update the 'database' of the quantity of tanks on each round

        round++;
        turn[round] = round;
        tank1_remain[round] = tanks1_qtty;
        tank2_remain[round] = tanks2_qtty;

        write_int (0, 950, 215 + 15 * round, 5, OFFSET turn[round]);
        write_int (0, 985, 215 + 15 * round, 5, OFFSET tank1_remain[round]);
        write_int (0, 1015, 215 + 15 * round, 5, OFFSET tank2_remain[round]);



//  ***********************************************************************
//
//  Now we add the firing of the group 1 - Very similar to
//  what we did for group 2

        FOR (g_ind = 1; g_ind <= tanks1_qtty; g_ind++)

            // Rotate the turret to aim to the current enemy

            angle2 = fget_angle ( tanks1_id[g_ind].x, tanks1_id[g_ind].y,
                                      tanks2_id[tanks2_qtty].x, tanks2_id[tanks2_qtty].y);

            // Gradually change the turret angle to point to the enemy tank

            turret_channel = sound (turret_snd, 512, 255);

            WHILE ( abs (tanks1_id[g_ind].son.angle - angle2) > ROTATE_ANGLE)

                tanks1_id[g_ind].son.angle = near_angle (tanks1_id[g_ind].son.angle, angle2, ROTATE_ANGLE);
                FRAME;
            END

            stop_sound (turret_channel);
            FRAME;

            sound(cannon_snd, 45, 192);


            FROM g_ind2 = 14 TO 16;
                tanks1_id[g_ind].son.graph = g_ind2;
                FRAME;
                FRAME;
            END

            tanks1_id[g_ind].son.graph = 12;

            tanks2_shield[tanks2_qtty]--;

            IF (tanks2_shield[tanks2_qtty] <= 0)

                // Tank destroyed - Draw big explosion

                sound (tank_expl_snd, 255, 255);

                big_explosion (tanks2_id[tanks2_qtty].x,
                                   tanks2_id[tanks2_qtty].y);
                signal (tanks2_id[tanks2_qtty], s_kill_tree);

                FRAME(1000);

                tanks2_qtty--;

            ELSE

                // Tanks not destroyed - draw 'hit'
                explosion(tanks2_id[tanks2_qtty].x + rand(-14,14),
                            tanks2_id[tanks2_qtty].y + rand(-7,7) );


            END

            WHILE (tanks2_qtty <= 0 )
                FRAME;
            END

            FRAME(fps*30);

        END


// Update the 'database' of the quantity of tanks on each round
        round++;
        turn[round] = round;
        tank1_remain[round] = tanks1_qtty;
        tank2_remain[round] = tanks2_qtty;

        write_int (0, 950, 215 + 15 * round, 5, OFFSET turn[round]);
        write_int (0, 985, 215 + 15 * round, 5, OFFSET tank1_remain[round]);
        write_int (0, 1015, 215 + 15 * round, 5, OFFSET tank2_remain[round]);

        FRAME;

    END
END



//-------------------------------------------------------------------
// Process tank1
// Draws the basis of a green tank
//-------------------------------------------------------------------


PROCESS tank1 (x, y)

PRIVATE

BEGIN

    z     = 0;
    graph = 10;


// It is important that the turret1() will be the last process to
// be called, so it will be accessed as ".son".  So we call shadow1 first,
// and then turret1
// Notice the coordinates offset of the shadow from the tank.
// (You could even simulate time advancing, by changing this offset
// to make this shadow smaller or bigger!)
//
//  The 'z' order of processes is:
//
// Upper    turret1 / turret2      -100
//          shadow of turret        -50  (shadow_t)
//          tank1 / tank2             0
// Lowest   shadow1 / shadow2        50
//
    shadow1 (x-5, y+4);


// The turret is called from the tank1 process. So 'turret1' is
// the son, and tank1 is the father process.

    turret1 (x+1, y);

    LOOP
        FRAME;
    END

END


//-------------------------------------------------------------------
// Process turret1
// Draws the turret of a green tank
//-------------------------------------------------------------------


PROCESS turret1 (x, y)

PRIVATE

BEGIN

// The turret has lower 'z' than the tank, so it will be always drawn
// OVER the tank

    z     = -100;
    graph = 12;


// Process to show the shadow of the turret

    shadow_t (x-5, y+4);


    LOOP
        FRAME;
    END

END


//-------------------------------------------------------------------
// Process shadow1
// Draws the shadow of the tanks (tank1)
//-------------------------------------------------------------------


PROCESS shadow1 (x, y)

PRIVATE

BEGIN

    z     = 50;
    graph = 11;

    LOOP
        FRAME;
    END

END



//-------------------------------------------------------------------
// Process tank2
// Draws the basis of a green tank
// Similar to tank1 - Only difference is orientation (angle = 180)
//-------------------------------------------------------------------


PROCESS tank2 (x, y)

PRIVATE

BEGIN

    z = 0;
    graph = 20;
    angle = 180000;

    shadow2 (x-5, y+4);
    turret2 (x-1, y+1);

    LOOP
        FRAME;
    END

END




//-------------------------------------------------------------------
// Process turret2
// Draws the turret of a gray tank
//-------------------------------------------------------------------


PROCESS turret2 (x, y)

PRIVATE

BEGIN

    z = -100;
    graph = 22;
    angle = 180000;

    shadow_t (x-5, y+4);

    LOOP
        FRAME;
    END

END


//-------------------------------------------------------------------
// Process shadow2
// Draws the shadow of the tanks (tank2)
//-------------------------------------------------------------------


PROCESS shadow2 (x, y)

PRIVATE

BEGIN

    z     = 50;
    graph = 21;

    LOOP
        FRAME;
    END

END



//-------------------------------------------------------------------
// Process shadow_t
// Draws the shadow of the turret
//-------------------------------------------------------------------


PROCESS shadow_t (x, y)

PRIVATE

BEGIN

    z     = -50;


// Since the turret can rotate, the shadow must go with it
// Also, during recoil, the turret graph changes, and so does
// its shadow

    LOOP
        graph = father.graph + 40;
        angle = father.angle;
        FRAME;
    END

END



//-------------------------------------------------------------------
// Process explosion
// Draws a small explosion (tank hit - not destroyed)
//-------------------------------------------------------------------

PROCESS explosion (x, y)

BEGIN

    z = -200;
    angle = rand(0,360000);

    FROM graph = 30 TO 32;
        FRAME;
    END

END


//-------------------------------------------------------------------
// Process big_explosion
// Draws a big explosion (tank destroyed)
//-------------------------------------------------------------------

PROCESS big_explosion (x, y)

BEGIN

    z = -200;

    FROM graph = 40 TO 44;
        FRAME;
    END

END

